@@ -18,7 +18,7 @@ use Test::BrewBuild::BrewCommands;
 use Test::BrewBuild::Dispatch;
 use Test::BrewBuild::Tester;
 
-our $VERSION = '2.02';
+our $VERSION = '2.03';
 
 my $log;
 my $bcmd;
@@ -44,7 +44,7 @@ sub options {
     my ($self, $args) = @_;
     my (%opts, $help, $setup);
 
-    _validate_opts($args);
+    my $bad_opt = _validate_opts($args);
 
     GetOptionsFromArray(
         $args,
@@ -60,12 +60,13 @@ sub options {
         "S|save"        => \$opts{save_reports},
         "l|legacy"      => \$opts{legacy},
         "T|selftest"    => \$opts{selftest},
+        "D|dispatch"    => \$opts{dispatch},
+        "t|tester=s@"   => \$opts{testers},
         "s|setup"       => \$setup,
         "h|help"        => \$help,
     );
 
-    help() if $help;
-    setup() if $setup;
+    $opts{error} = 1 if $bad_opt;
 
     return %opts;
 }
@@ -92,33 +93,55 @@ sub brew_info {
     return $brew_info;
 }
 sub perls_available {
-    my ($self, $brew_info) = @_;
+    my $self = shift;
     my $log = $log->child('perls_available');
-    my @perls_available = $bcmd->available($self->legacy, $brew_info);
+    my @perls_available = $bcmd->available($self->legacy, $self->brew_info);
     $log->_6("perls available: " . join ', ', @perls_available);
     return @perls_available;
 }
 sub perls_installed {
-    my ($self, $brew_info) = @_;
+    my $self = shift;
     my $log = $log->child('perls_installed');
     $log->_6("checking perls installed");
-    return $bcmd->installed($self->legacy, $brew_info);
+    return $bcmd->installed($self->legacy, $self->brew_info);
 }
 sub instance_install {
-    my ($self, $install) = @_;
+    my ($self, $install, $timeout) = @_;
+
+    # timeout an install after...
+
+    $timeout = $timeout || 300;
 
     my $log = $log->child('instance_install');
 
-    my $brew_info = $self->brew_info;
-
-    my @perls_available = $self->perls_available($brew_info);
-    my @perls_installed = $self->perls_installed($brew_info);
+    my @perls_available = $self->perls_available;
+    my @perls_installed = $self->perls_installed;
     my @new_installs;
 
     if (ref $install eq 'ARRAY'){
         for my $version (@$install){
             $version = "perl-$version" if ! $self->is_win && $version !~ /perl/;
-            $version .= '_64' if $self->is_win && $version !~ /_/;
+
+            if ($self->is_win){
+                if ($version !~ /_/){
+                    $log->_7("MSWin: no bit suffix supplied");
+                    if (! grep {$_ =~ /$version/} @perls_available){
+                        $version .= '_64';
+                        $log->_7("MSWin: default to available 64-bit $version");
+                    }
+                    else {
+                        $version .= '_32';
+                        $log->_7("MSWin: no 64-bit version... using $version");
+                    }
+                }
+
+            }
+            $version =~ s/_.*$// if ! $self->is_win;
+
+            if (! grep { $version eq $_ } @perls_available){
+                $log->_0("$version is not a valid perl version");
+                next;
+            }
 
             if (grep { $version eq $_ } @perls_installed){
                 $log->_6("$version is already installed... skipping");
@@ -159,47 +182,70 @@ sub instance_install {
 
         my $install_cmd = $bcmd->install;
 
-        for my $ver (@new_installs){
-            $log->_0("installing $ver...");
-            $log->_5("...using cmd: $install_cmd");
-            `$install_cmd $ver`;
+        for my $ver (@new_installs) {
+            $log->_0( "installing $ver..." );
+            $log->_5( "...using cmd: $install_cmd" );
+            eval {
+                local $SIG{ALRM} = sub {
+                    die "$ver failed to install... skipping"
+                };
+                alarm $timeout;
+                `$install_cmd $ver`;
+                alarm 0;
+            };
+            if ($@){
+                $log->_0($@);
+                $self->instance_remove($ver);
+                next;
+            }
         }
     }
     else {
-        $log->_5("using existing versions only");
+        $log->_5("using existing versions only, nothing to install");
     }
 }
 sub instance_remove {
-    my $self = shift;
+    my ($self, $version) = @_;
 
     my $log = $log->child('instance_remove');
 
-    my @perls_installed = $self->perls_installed($self->brew_info);
+    my @perls_installed = $self->perls_installed;
 
     $log->_6("perls installed: " . join ', ', @perls_installed);
-    $log->_0("removing previous installs...");
 
     my $remove_cmd = $bcmd->remove;
 
     $log->_4( "using '$remove_cmd' remove command" );
 
-    for my $installed_perl (@perls_installed){
-
-        my $using = $bcmd->using( $self->brew_info );
-
-        if ($using eq $installed_perl) {
-            $log->_5( "not removing version we're using: $using" );
-            next;
-        }
-
-        $log->_5( "exec'ing $remove_cmd $installed_perl" );
-
-        if ($bcmd->is_win) {
-            `$remove_cmd $installed_perl 2>nul`;
+    if ($version){
+        $log->_5("$version supplied, removing...");
+        if ($self->is_win){
+            `$remove_cmd $version 2`
         }
         else {
-            `$remove_cmd $installed_perl 2>/dev/null`;
+            `$remove_cmd $version 2>/dev/null`;
+        }
+    }
+    else {
+        $log->_0("removing previous installs...");
 
+        for my $installed_perl (@perls_installed){
+
+            my $using = $bcmd->using($self->brew_info);
+
+            if ($using eq $installed_perl) {
+                $log->_5( "not removing version we're using: $using" );
+                next;
+            }
+
+            $log->_5( "exec'ing $remove_cmd $installed_perl" );
+
+            if ($bcmd->is_win) {
+                `$remove_cmd $installed_perl 2>nul`;
+            }
+            else {
+                `$remove_cmd $installed_perl 2>/dev/null`;
+            }
         }
     }
 
@@ -392,6 +438,11 @@ Local usage options:
 -N | --notest   Do not run tests. Allows you to --remove and --install without testing
 -l | --legacy   Operate on perls < 5.8.x. The default plugins won't work with this flag set if a lower version is installed
 
+Network dispatching options:
+
+-D | --dispatch Dispatch a basic run to remote testers
+-t | --tester   Testers to dispatch to. Can be supplied multiple times. Format: "host[:port]"
+
 Help options:
 
 -s | --setup    Display test platform setup instructions
@@ -405,7 +456,7 @@ Special options:
 -d | --debug    0-7, sets logging verbosity, default is 0
 
 EOF
-exit;
+return 1;
 }
 sub _attach_build_log {
     # attach the cpanm logs to the PASS/FAIL logs
@@ -746,9 +797,8 @@ sub _validate_opts {
 
     my @valid_args = qw(
         on o new n remove r revdep R plugin p args a debug d install i help h
-        N notest setup s legacy l selftest T listen L
-        tester-port t testers S save
-        );
+        N notest setup s legacy l selftest T t testers S save D dispatch
+    );
 
     my $bad_opt = 0;
 
@@ -762,8 +812,7 @@ sub _validate_opts {
             }
         }
     }
-
-    help() if $bad_opt;
+    return $bad_opt;
 }
 
 1;
@@ -854,6 +903,18 @@ If an integer is sent in, we'll install that many random versions of perl. You
 can also send in an array reference, where each element is a version of perl,
 and we'll install those instead.
 
+You can send a second parameter, an integer for a time out. On each install,
+we'll bail if it takes longer than this time. Default is 300 seconds. If you're
+on a fast machine, you should probably lower this value.
+
+On Windows, where you want to install specific perls, we'll default to
+installing 64-bit versions only, if a 64 bit perl is available for the version
+desired and you haven't added the C<_64/_32> suffix per C<berrybrew available>.
+
+Simply add the C<_32> suffix if you want to install it specifically. Note that
+if you're dispatching to Unix and Windows servers, the Unix systems will remove
+this invalid portion of the version prior to processing further.
+
 =head2 instance_remove
 
 Uninstalls all currently installed perls, less the one you are currently
@@ -887,6 +948,9 @@ a hash of the translated C<Test::BrewBuild> parameters along with their values.
 
 Returns the converted hash for passing back into C<new()>.
 
+If an invalid argument is included, we'll set C<$args{error} = 1;>. It is up to
+the caller to look for and process an error handling routine.
+
 =head2 config_file
 
 Returns a string that contains the path/filename of the configuration file, if
