@@ -4,16 +4,26 @@ use warnings;
 
 use Capture::Tiny qw(:all);
 
-our $VERSION = '2.02';
+our $VERSION = '2.03';
 
 sub new {
     return bless {}, shift;
 }
 sub git {
     my $self = shift;
-    my $cmd = $^O =~ /MSWin/
-        ? (split /\n/, `where git.exe`)[0]
-        : 'git';
+    my $cmd;
+
+    if ($^O =~ /MSWin/){
+        for (split /;/, $ENV{PATH}){
+            if (-x "$_/git.exe"){
+                $cmd = "$_/git.exe";
+                last;
+            }
+        }
+    }
+    else {
+        $cmd = 'git';
+    }
     return $cmd;
 }
 sub link {
