@@ -13,7 +13,7 @@ use Storable;
 use Test::BrewBuild;
 use Test::BrewBuild::Git;
 
-our $VERSION = '2.02';
+our $VERSION = '2.03';
 
 $| = 1;
 
@@ -68,9 +68,20 @@ sub start {
     if ($^O =~ /MSWin/){
         $log->_6("on Windows, using work dir $work_dir");
 
-        $perl = (split /\n/, `where perl.exe`)[0];
-        my $t = (split /\n/, `where bbtester`)[0];
+        my $t;
 
+        for (split /;/, $ENV{PATH}){
+            if (-x "$_/perl.exe"){
+                $perl = "$_/perl.exe";
+                last;
+            }
+        }
+        for (split /;/, $ENV{PATH}){
+            if (-e "$_/bbtester"){
+                $t = "$_/bbtester";
+                last;
+            }
+        }
         $log->_6("using command: $perl $t --fg");
 
         @args = ($t, '--fg');
@@ -220,7 +231,8 @@ sub listen {
         my @args = split /\s+/, $cmd;
 
         if ($args[0] ne 'brewbuild'){
-            my $err = "error: only 'brewbuild' is allowed as a command\n\n";
+            my $err = "error: only 'brewbuild' is allowed as a command. ";
+            $err .= "you sent in: " . join ' ', @args;
             $log->_0($err);
             $dispatch->send($err);
             next;
@@ -254,13 +266,25 @@ sub listen {
             }
 
             my %opts = Test::BrewBuild->options(\@args);
+
+            if (defined $opts{error}){
+                my $err = "invalid arguments sent to brewbuild: ";
+                $err .= join ', ', @args;
+                $log->_0($err);
+                $dispatch->send($err);
+                next;
+            }
             my $opt_str;
 
             for (keys %opts){
                 $opt_str .= "$_ => $opts{$_}\n" if defined $opts{$_};
             }
-            $log->_5("commencing test run with args: $opt_str") if $opt_str;
-            $log->_5("commencing default brewbuild run with no args");
+            if ($opt_str){
+                $log->_5("commencing test run with args: $opt_str");
+            }
+            else {
+                $log->_5("commencing default brewbuild run with no args");
+            }
 
             my $bb = Test::BrewBuild->new(%opts);
             $bb->log()->file(\$self->{log}) if ! $self->{log_to_stdout};
